<?php

namespace Drupal\monolog_extra\Logger\Handler;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Monolog\Handler\RotatingFileHandler;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Monolog\Logger;
use Monolog\LogRecord;

/**
 * Safely creates the file handlers.
 *
 * In order to make the handler secure, it does:
 *   - Prevent fatal errors.
 *   - Auto creates the log folder in case it does not exist.
 *   - Logs in case logs folder can't be generated.
 */
class SafeRotatingFileHandler extends RotatingFileHandler {

  use StringTranslationTrait;

  /**
   * File system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $loggerChannel;

  /**
   * Error message when log file can't be written.
   *
   * @var string
   */
  public static $errorMessage;

  /**
   * SafeRotatingFileHandler constructor.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   File system.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $loggerChannel
   *   Logger.
   * @param string $path
   *   Path where the log files will be stored.
   * @param int $maxFiles
   *   Number of maximum files. Optional.
   */
  public function __construct(FileSystemInterface $fileSystem, LoggerChannelInterface $loggerChannel, string $path, int $maxFiles = 0) {
    $this->fileSystem = $fileSystem;
    $this->loggerChannel = $loggerChannel;
    parent::__construct($path, $maxFiles, Logger::DEBUG, TRUE, 0664);
  }

  /**
   * Check that logs directory exists.
   *
   * @return bool
   *   Directory logs exists.
   */
  public function prepareLogDirectory() {
    $dir = dirname($this->filename);
    if ($this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY)) {
      $directory_ready = TRUE;
    }
    else {
      $this->loggerChannel->warning('Directory "%dir" for logs not exists or does not have the right permissions, please revise it.', ['%dir' => $dir]);
      $directory_ready = FALSE;
    }
    return $directory_ready;
  }

  /**
   * Check the handler can write.
   *
   * @return bool
   *   TRUE if logger can write logs.
   */
  protected function canWrite() {
    return $this->prepareLogDirectory() && $this->logFileCanBeCreated();
  }

  /**
   * Check the log file can be created.
   *
   * @return bool
   *   TRUE when log file can be created.
   */
  public function logFileCanBeCreated() {
    $url = $this->getUrl();
    @fopen($url, 'a');
    set_error_handler([$this, 'customErrorHandler']);
    $stream = fopen($url, 'a');
    restore_error_handler();
    if (is_resource($stream)) {
      $can_be_created = TRUE;
    }
    else {
      $can_be_created = FALSE;
      $this->loggerChannel->critical($this->t('The stream or file :file could not be opened: :message', [
        ':file' => $url,
        ':message' => self::$errorMessage,
      ]));
    }
    return $can_be_created;
  }

  /**
   * Custom handler to get the warning on open file.
   *
   * @param string $msg
   *   Message error.
   */
  public function customErrorHandler($msg) {
    self::$errorMessage = preg_replace('{^(fopen|mkdir)\(.*?\): }', '', $msg);
  }

  /**
   * {@inheritdoc}
   */
  protected function write(LogRecord $record) : void {
    if ($this->canWrite()) {
      parent::write($record);
    }
  }

}
